/*
 * file.c
 *
 * This file contains some simple io functions
 *
 * Started 4/10/95
 * $Id: file.c,v 1.6 2002/07/24 16:34:05 karypis Exp $
 *
 */

#include <GKlib.h>

/*************************************************************************
* This function opens a file
**************************************************************************/
FILE *GKfopen(char *fname, char *mode, char *msg)
{
  FILE *fp;
  char errmsg[256];

  fp = fopen(fname, mode);
  if (fp != NULL)
    return fp;

  sprintf(errmsg,"file: %s, mode: %s, [%s]", fname, mode, msg);
  perror(msg);
  errexit("Failed on GKfopen\n");
}


/*************************************************************************
* This function closes a file
**************************************************************************/
void GKfclose(FILE *fp)
{
  fclose(fp);
}


/*************************************************************************
* This function checks if a file exists
**************************************************************************/
int GKfexist(char *fname)
{
  FILE *fp;

  if ((fp = fopen(fname, "r")) == NULL)
    return 0;
    
  fclose(fp);
  return 1;
}


/*************************************************************************
* This function gets some basic statistics about the file
**************************************************************************/
void GKGetFileStats(char *fname, int *r_nlines, int *r_ntokens, int *r_nbytes)
{
  int nlines, ntokens, nbytes;
  FILE *fpin;
  char *line, delim[] = " \t", *token;

  line = (char *)GKmalloc(1024*1024*sizeof(char), "GKGetFileStats: line");

  fpin = GKfopen(fname, "r", "GKGetFileStats");

  nlines = ntokens = nbytes = 0;

  while (fgets(line, 1024*1024-1, fpin)) {
    nlines++;
    nbytes += strlen(line);

    token = strtok(line, delim);
    while (token) {
      ntokens++;
      token = strtok(NULL, delim);
    }
  }

  GKfclose(fpin);

  *r_nlines  = nlines;
  *r_ntokens = ntokens;
  *r_nbytes  = nbytes;

  free(line);
}
