/*
 * Copyright 1997, Regents of the University of Minnesota
 *
 * memory.c
 *
 * This function contains various utility routines
 *
 * Started 9/28/95
 * George
 *
 * $Id: memory.c,v 1.12 2004/11/26 18:58:08 karypis Exp $
 */

#include <GKlib.h>

volatile static int GKlibMaxMemory      = 0;
volatile static int GKlibCurrentMemory  = 0;


#ifndef DMALLOC
/*************************************************************************
* The following function allocates an array of integers
**************************************************************************/
int *imalloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (int *)GKmalloc(sizeof(int)*n, msg);
}



/*************************************************************************
* The following function allocates an array of float 
**************************************************************************/
float *fmalloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (float *)GKmalloc(sizeof(float)*n, msg);
}


/*************************************************************************
* The following function allocates an array of float 
**************************************************************************/
double *dmalloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (double *)GKmalloc(sizeof(double)*n, msg);
}


/*************************************************************************
* The following function allocates an array of float 
**************************************************************************/
KeyValueType *keymalloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (KeyValueType *)GKmalloc(sizeof(KeyValueType)*n, msg);
}



/*************************************************************************
* The following function allocates an array of float 
**************************************************************************/
FKeyValueType *fkeymalloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (FKeyValueType *)GKmalloc(sizeof(FKeyValueType)*n, msg);
}


FKeyValueType2 *fkey2malloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (FKeyValueType2 *)GKmalloc(sizeof(FKeyValueType2)*n, msg);
}

/*************************************************************************
* The following function allocates an array of float 
**************************************************************************/
DKeyValueType *dkeymalloc(int n, char *msg)
{
  if (n == 0)
    return NULL;

  return (DKeyValueType *)GKmalloc(sizeof(DKeyValueType)*n, msg);
}


/*************************************************************************
* The follwoing function allocates an array of integers
**************************************************************************/
int *ismalloc(int n, int ival, char *msg)
{
  if (n == 0)
    return NULL;

  return iset(n, ival, (int *)GKmalloc(sizeof(int)*n, msg));
}


/*************************************************************************
* The follwoing function allocates an array of floats
**************************************************************************/
float *fsmalloc(int n, float ival, char *msg)
{
  if (n == 0)
    return NULL;

  return fset(n, ival, (float *)GKmalloc(sizeof(float)*n, msg));
}


/*************************************************************************
* The follwoing function allocates an array of floats
**************************************************************************/
double *dsmalloc(int n, double ival, char *msg)
{
  if (n == 0)
    return NULL;

  return dset(n, ival, (double *)GKmalloc(sizeof(double)*n, msg));
}



/*************************************************************************
* This function allocates a two-dimensional matrix 
**************************************************************************/
void GKAllocMatrix(void ***r_matrix, int elmlen, int ndim1, int ndim2)
{
  int i;
  void **matrix;


  matrix = (void **)GKmalloc(ndim1*sizeof(void *), "GKAllocMatrix: matrix");
  for (i=0; i<ndim1; i++) 
    matrix[i] = (void *)GKmalloc(ndim2*elmlen, "GKAllocMatrix: matrix[i]");

  *r_matrix = matrix;
}


/*************************************************************************
* This function frees a two-dimensional matrix 
**************************************************************************/
void GKFreeMatrix(void **matrix, int ndim1, int ndim2)
{
  int i;

  for (i=0; i<ndim1; i++) 
    GKfree((void *)&matrix[i], LTERM);

  GKfree((void *)&matrix, LTERM);

}


/*************************************************************************
* This function allocates a two-dimensional matrix 
**************************************************************************/
int **iAllocMatrix(int ndim1, int ndim2, int value, char *errmsg)
{
  int i;
  int **matrix;

  matrix = (int **)GKmalloc(ndim1*sizeof(int *), errmsg);
  for (i=0; i<ndim1; i++) 
    matrix[i] = ismalloc(ndim2, value, errmsg);

  return matrix;
}


/*************************************************************************
* This function frees a two-dimensional matrix 
**************************************************************************/
void iFreeMatrix(int ***r_matrix, int ndim1, int ndim2)
{
  int i;
  int **matrix;

  matrix = *r_matrix;

  for (i=0; i<ndim1; i++) 
    GKfree((void *)&(matrix[i]), LTERM);

  GKfree((void *)&matrix, LTERM);

  *r_matrix = NULL;

}


/*************************************************************************
* This function allocates a two-dimensional matrix 
**************************************************************************/
float **fAllocMatrix(int ndim1, int ndim2, float value, char *errmsg)
{
  int i;
  float **matrix;

  matrix = (float **)GKmalloc(ndim1*sizeof(float *), errmsg);
  for (i=0; i<ndim1; i++) 
    matrix[i] = fsmalloc(ndim2, value, errmsg);

  return matrix;
}


/*************************************************************************
* This function frees a two-dimensional matrix 
**************************************************************************/
void fFreeMatrix(float ***r_matrix, int ndim1, int ndim2)
{
  int i;
  float **matrix;

  matrix = *r_matrix;

  for (i=0; i<ndim1; i++) 
    GKfree((void *)&(matrix[i]), LTERM);

  GKfree((void *)&matrix, LTERM);

  *r_matrix = NULL;

}


/*************************************************************************
* This function allocates a two-dimensional matrix 
**************************************************************************/
char **cAllocMatrix(int ndim1, int ndim2, char value, char *errmsg)
{
  int i, j;
  char **matrix;

  matrix = (char **)GKmalloc(ndim1*sizeof(char *), errmsg);
  for (i=0; i<ndim1; i++) {
    matrix[i] = (char *)GKmalloc(ndim2*sizeof(char), errmsg);
    for (j=0; j<ndim2; j++)
      matrix[i][j] = value;
  }

  return matrix;
}


/*************************************************************************
* This function frees a two-dimensional matrix 
**************************************************************************/
void cFreeMatrix(char ***r_matrix, int ndim1, int ndim2)
{
  int i;
  char **matrix;

  matrix = *r_matrix;

  for (i=0; i<ndim1; i++) 
    GKfree((void *)&(matrix[i]), LTERM);

  GKfree((void *)&matrix, LTERM);

  *r_matrix = NULL;

}

/*************************************************************************
* This function fills-in a 2D matrix of characters
**************************************************************************/
void cSetMatrix(char **matrix, int ndim1, int ndim2, char value)
{
  int i, j;

  for (i=0; i<ndim1; i++) {
    for (j=0; j<ndim2; j++)
      matrix[i][j] = value;
  }
}



/*************************************************************************
* This function is my wrapper around malloc
**************************************************************************/
void *GKmalloc(int nbytes, char *msg)
{
  void *ptr;

  if (nbytes == 0)
    return NULL;

#ifdef GKMEMSTATS
  ptr = (void *)malloc(nbytes + sizeof(size_t));
  if (ptr == NULL) 
    errexit("***Memory allocation failed for %s. Requested size: %d bytes", msg, nbytes);
  
  *((size_t *)ptr) = nbytes;
  GKlibCurrentMemory += nbytes;
  GKlibMaxMemory = (GKlibCurrentMemory > GKlibMaxMemory ? GKlibCurrentMemory : GKlibMaxMemory);
 
  return (void *)((size_t *) ptr + 1);
#else
  ptr = (void *)malloc(nbytes);
  if (ptr == NULL)
    errexit("***Memory allocation failed for %s. Requested size: %d bytes", msg, nbytes);

  return ptr;
#endif
}
#endif


/*************************************************************************
* This function is my wrapper around free, allows multiple pointers    
**************************************************************************/
void GKfree(void **ptr1,...)
{
  va_list plist;
  void **ptr, *mptr;

#ifdef GKMEMSTATS
  if (*ptr1 != NULL) {
    mptr = *ptr1;
    GKlibCurrentMemory -= *((size_t *)mptr - 1);
    free((size_t *)mptr-1);
  }
  *ptr1 = NULL;

  va_start(plist, ptr1);

  /* while ((int)(ptr = va_arg(plist, void **)) != -1) { */
  while ((ptr = va_arg(plist, void **)) != LTERM) {
    if (*ptr != NULL) {
      mptr = *ptr;
      GKlibCurrentMemory -= *((size_t *)mptr - 1);
      free((size_t *)mptr-1);
    }
    *ptr = NULL;
  }
#else
  if (*ptr1 != NULL)
    free(*ptr1);
  *ptr1 = NULL;

  va_start(plist, ptr1);

  while ((ptr = va_arg(plist, void **)) != LTERM) {
    if (*ptr != NULL)
      free(*ptr);
    *ptr = NULL;
  }
#endif

  va_end(plist);
}            

/*************************************************************************
* This function returns the maximum memory alocated on the heap
**************************************************************************/
int GKResetMemoryUsage()
{
  GKlibMaxMemory     = 0;
  GKlibCurrentMemory = 0;
}

/*************************************************************************
* This function returns the maximum memory alocated on the heap
**************************************************************************/
int GKGetMaxMemoryUsed()
{
  return GKlibMaxMemory;
}
