/*
 * tokenizer.c
 *
 * This file contains various routines for splitting an input string into
 * tokens and returning them in form of a list. The goal is to mimic
 * perl's split function
 *
 * Started 11/23/04
 * George
 *
 * $Id: tokenizer.c,v 1.1 2004/11/26 16:02:08 karypis Exp $
 *
 */

#include <GKlib.h>


/************************************************************************
* This function tokenizes a string based on the user-supplied delimiters
* list. The resulting tokens are returned into an array of strings.
*************************************************************************/
void strtokenize(char *str, char *delim, TokensType *tokens)
{
  int i, j, ntoks, slen;

  tokens->strbuf = strdup(str);

  slen  = strlen(str);
  str   = tokens->strbuf;

  /* Scan once to determine the number of tokens */
  for (ntoks=0, i=0; i<slen;) {
    /* Consume all the consecutive characters from the delimiters list */
    while (i<slen && strchr(delim, str[i])) 
      i++;

    if (i == slen)
      break;

    ntoks++;

    /* Consume all the consecutive characters from the token */
    while (i<slen && !strchr(delim, str[i])) 
      i++;
  }


  tokens->ntoks = ntoks;
  tokens->list  = (char **)GKmalloc(ntoks*sizeof(char *), "strtokenize: tokens->list");


  /* Scan a second time to mark and link the tokens */
  for (ntoks=0, i=0; i<slen;) {
    /* Consume all the consecutive characters from the delimiters list */
    while (i<slen && strchr(delim, str[i])) 
      str[i++] = '\0';

    if (i == slen)
      break;

    tokens->list[ntoks++] = str+i;

    /* Consume all the consecutive characters from the token */
    while (i<slen && !strchr(delim, str[i])) 
      i++;
  }
}


/************************************************************************
* This function frees the memory associated with a TokensType
*************************************************************************/
void freetokenslist(TokensType *tokens)
{
  GKfree((void *)&tokens->list, &tokens->strbuf, LTERM);
}

