/*
 * Copyright 1997, Regents of the University of Minnesota
 *
 * util.c
 *
 * This file contains utility functions
 *
 * $Id: util.c,v 1.10 2003/11/09 12:03:21 karypis Exp $
 *
 */

#include <GKlib.h>



/*************************************************************************
* This function prints an error message and exits
**************************************************************************/
void *errexit(char *f_str,...)
{
  va_list argp;

  va_start(argp, f_str);
  vfprintf(stderr, f_str, argp);
  va_end(argp);

  fprintf(stderr,"\n");
  fflush(stderr);

  exit(-1);
}



/*************************************************************************
* This file randomly permutes the contents of an array.
* flag == 0, don't initialize perm
* flag == 1, set p[i] = i 
**************************************************************************/
void RandomPermute(int n, int *p, int flag)
{
  int i, u, v, tmp, m;

  if (flag == 1)
    for (i=0; i<n; i++)
      p[i] = i;

  m = n-8;
  for (i=0; i<n/16; i++) {
    v = RandomInRange(m);
    u = RandomInRange(m);
    SWAP(p[v], p[u], tmp);
    SWAP(p[v+1], p[u+1], tmp);
    SWAP(p[v+2], p[u+2], tmp);
    SWAP(p[v+3], p[u+3], tmp);
  }
}


/*************************************************************************
* This file randomly permutes the contents of an array.
* flag == 0, don't initialize perm
* flag == 1, set p[i] = i 
**************************************************************************/
void RandomPermuteCoarse(int n, int *p, int flag)
{
  int i, u, v, tmp, m;

  if (flag == 1)
    for (i=0; i<n; i++)
      p[i] = i;

  m = n-8;
  for (i=0; i<n/10; i++) {
    v = RandomInRange(m);
    u = RandomInRange(m);
    SWAP(p[v], p[u], tmp);
  }
}


/*************************************************************************
* This file randomly permutes the contents of an array.
* flag == 0, don't initialize perm
* flag == 1, set p[i] = i 
**************************************************************************/
void RandomPermuteFine(int n, int *p, int flag)
{
  int i, u, v, tmp, m;

  if (flag == 1)
    for (i=0; i<n; i++)
      p[i] = i;

  for (i=0; i<n; i++) {
    v = RandomInRange(n);
    u = RandomInRange(n);
    SWAP(p[v], p[u], tmp);
  }
}



/*************************************************************************
* This function returns the log2(x)
**************************************************************************/
int mylog2(int a)
{
  int i;

  for (i=1; a > 1; i++, a = a>>1);
  return i-1;
}


/*************************************************************************
* This function checks if the argument is a power of 2
**************************************************************************/
int ispow2(int a)
{
  return (a == (1<<mylog2(a)));
}


/*************************************************************************
* This function returns the log2(x)
**************************************************************************/
float flog2(float a)
{
  return log(a)/log(2.0);
}



/*************************************************************************
* This function returns the seconds
**************************************************************************/
double seconds(void)
{
#ifdef _MSC_VER
  return((double) clock()/CLOCKS_PER_SEC);
#else
  struct rusage r;

  getrusage(RUSAGE_SELF, &r);
  return ((r.ru_utime.tv_sec + r.ru_stime.tv_sec) + 1.0e-6*(r.ru_utime.tv_usec + r.ru_stime.tv_usec));
#endif
}

