/*
 * matrices.c
 *
 * This file contains various AA-scoring matrices as well as the AA-order to be 
 * used in the program. This information is copied into the appropriate fields 
 * of the ctrl structure.
 *
 * Started 11/22/04
 * George
 *
 * $Id: matrices.c,v 1.12 2004/12/29 15:35:59 karypis Exp $
 *
 */

#include <aasim.h>

/* The order of the amino acids in the scoring matrix. Adheres to NCBI */
static char *AAORDER = "ARNDCQEGHILKMFPSTWYVBZX*";

static int FREQBLOSUM62[] = {
/*     A    R    N    D    C    Q    E    G    H    I    L    K    M    F    P    S    T    W    Y    V  */
/*A*/ 29,   3,   2,   2,   2,   2,   4,   7,   1,   4,   5,   4,   1,   2,   2,   8,   4,   0,   1,   6,
/*R*/  4,  34,   3,   3,   0,   4,   5,   3,   2,   2,   4,  12,   1,   1,   1,   4,   3,   0,   1,   3,
/*N*/  4,   4,  31,   8,   0,   3,   4,   6,   3,   2,   3,   5,   1,   1,   2,   6,   4,   0,   1,   2,
/*D*/  4,   2,   6,  39,   0,   2,   9,   4,   1,   2,   2,   4,   0,   1,   2,   5,   3,   0,   1,   2,
/*C*/  6,   1,   1,   1,  48,   1,   1,   3,   0,   4,   6,   2,   1,   2,   1,   4,   3,   0,   1,   5,
/*Q*/  5,   7,   4,   4,   0,  21,  10,   4,   2,   2,   4,   9,   2,   1,   2,   5,   4,   0,   2,   3,
/*E*/  5,   4,   4,   9,   0,   6,  29,   3,   2,   2,   3,   7,   1,   1,   2,   5,   3,   0,   1,   3,
/*G*/  7,   2,   3,   3,   1,   1,   2,  51,   1,   1,   2,   3,   0,   1,   1,   5,   2,   0,   1,   2,
/*H*/  4,   4,   5,   3,   0,   3,   5,   3,  35,   2,   3,   4,   1,   3,   1,   4,   2,   0,   5,   2,
/*I*/  4,   1,   1,   1,   1,   1,   1,   2,   0,  27,  16,   2,   3,   4,   1,   2,   3,   0,   2,  17,
/*L*/  4,   2,   1,   1,   1,   1,   2,   2,   1,  11,  37,   2,   4,   5,   1,   2,   3,   0,   2,   9,
/*K*/  5,  10,   4,   4,   0,   5,   7,   4,   2,   2,   4,  27,   1,   1,   2,   5,   3,   0,   1,   3,
/*M*/  5,   3,   2,   2,   1,   2,   2,   2,   1,  10,  19,   3,  16,   4,   1,   3,   4,   0,   2,   9,
/*F*/  3,   1,   1,   1,   1,   1,   1,   2,   1,   6,  11,   1,   2,  38,   1,   2,   2,   1,   8,   5,
/*P*/  5,   2,   2,   3,   1,   2,   3,   3,   1,   2,   3,   4,   1,   1,  49,   4,   3,   0,   1,   3,
/*S*/ 10,   4,   5,   4,   1,   3,   5,   6,   1,   2,   4,   5,   1,   2,   2,  21,   8,   0,   1,   4,
/*T*/  7,   3,   4,   3,   1,   2,   3,   4,   1,   5,   6,   4,   1,   2,   2,   9,  24,   0,   1,   7,
/*W*/  3,   2,   1,   1,   0,   1,   2,   3,   1,   3,   5,   2,   1,   6,   0,   2,   2,  49,   6,   3,
/*Y*/  4,   2,   2,   1,   0,   2,   2,   2,   4,   4,   6,   3,   1,  13,   1,   3,   2,   2,  31,   4,
/*V*/  6,   2,   1,   1,   1,   1,   2,   2,   0,  16,  13,   2,   3,   3,   1,   3,   4,   0,   2,  26
};


/* BLOSUM62 matrix */
static int BLOSUM62[] = {
/*     A   R   N   D   C   Q   E   G   H   I   L   K   M   F   P   S   T   W   Y   V   B   Z   X   * */
/*A*/ +4, -1, -2, -2, +0, -1, -1, +0, -2, -1, -1, -1, -1, -2, -1, +1, +0, -3, -2, +0, -2, -1, +0, -4,
/*R*/ -1, +5, +0, -2, -3, +1, +0, -2, +0, -3, -2, +2, -1, -3, -2, -1, -1, -3, -2, -3, -1, +0, -1, -4,
/*N*/ -2, +0, +6, +1, -3, +0, +0, +0, +1, -3, -3, +0, -2, -3, -2, +1, +0, -4, -2, -3, +3, +0, -1, -4,
/*D*/ -2, -2, +1, +6, -3, +0, +2, -1, -1, -3, -4, -1, -3, -3, -1, +0, -1, -4, -3, -3, +4, +1, -1, -4,
/*C*/ +0, -3, -3, -3, +9, -3, -4, -3, -3, -1, -1, -3, -1, -2, -3, -1, -1, -2, -2, -1, -3, -3, -2, -4,
/*Q*/ -1, +1, +0, +0, -3, +5, +2, -2, +0, -3, -2, +1, +0, -3, -1, +0, -1, -2, -1, -2, +0, +3, -1, -4,
/*E*/ -1, +0, +0, +2, -4, +2, +5, -2, +0, -3, -3, +1, -2, -3, -1, +0, -1, -3, -2, -2, +1, +4, -1, -4,
/*G*/ +0, -2, +0, -1, -3, -2, -2, +6, -2, -4, -4, -2, -3, -3, -2, +0, -2, -2, -3, -3, -1, -2, -1, -4,
/*H*/ -2, +0, +1, -1, -3, +0, +0, -2, +8, -3, -3, -1, -2, -1, -2, -1, -2, -2, +2, -3, +0, +0, -1, -4,
/*I*/ -1, -3, -3, -3, -1, -3, -3, -4, -3, +4, +2, -3, +1, +0, -3, -2, -1, -3, -1, +3, -3, -3, -1, -4,
/*L*/ -1, -2, -3, -4, -1, -2, -3, -4, -3, +2, +4, -2, +2, +0, -3, -2, -1, -2, -1, +1, -4, -3, -1, -4,
/*K*/ -1, +2, +0, -1, -3, +1, +1, -2, -1, -3, -2, +5, -1, -3, -1, +0, -1, -3, -2, -2, +0, +1, -1, -4,
/*M*/ -1, -1, -2, -3, -1, +0, -2, -3, -2, +1, +2, -1, +5, +0, -2, -1, -1, -1, -1, +1, -3, -1, -1, -4,
/*F*/ -2, -3, -3, -3, -2, -3, -3, -3, -1, +0, +0, -3, +0, +6, -4, -2, -2, +1, +3, -1, -3, -3, -1, -4,
/*P*/ -1, -2, -2, -1, -3, -1, -1, -2, -2, -3, -3, -1, -2, -4, +7, -1, -1, -4, -3, -2, -2, -1, -2, -4,
/*S*/ +1, -1, +1, +0, -1, +0, +0, +0, -1, -2, -2, +0, -1, -2, -1, +4, +1, -3, -2, -2, +0, +0, +0, -4,
/*T*/ +0, -1, +0, -1, -1, -1, -1, -2, -2, -1, -1, -1, -1, -2, -1, +1, +5, -2, -2, +0, -1, -1, +0, -4,
/*W*/ -3, -3, -4, -4, -2, -2, -3, -2, -2, -3, -2, -3, -1, +1, -4, -3, -2,+11, +2, -3, -4, -3, -2, -4,
/*Y*/ -2, -2, -2, -3, -2, -1, -2, -3, +2, -1, -1, -2, -1, +3, -3, -2, -2, +2, +7, -1, -3, -2, -1, -4,
/*V*/ +0, -3, -3, -3, -1, -2, -2, -3, -3, +3, +1, -2, +1, -1, -2, -2, +0, -3, -1, +4, -3, -2, -1, -4,
/*B*/ -2, -1, +3, +4, -3, +0, +1, -1, +0, -3, -4, +0, -3, -3, -2, +0, -1, -4, -3, -3, +4, +1, -1, -4,
/*Z*/ -1, +0, +0, +1, -3, +3, +4, -2, +0, -3, -3, +1, -1, -3, -1, +0, -1, -3, -2, -2, +1, +4, -1, -4,
/*X*/ +0, -1, -1, -1, -2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2, +0, +0, -2, -1, -1, -1, -1, -1, -4,
/***/ -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, -4, +1};


/**************************************************************************************
* This function copies the information from the various matrices into the corresponding
* two dimensional arrays of the control structure
***************************************************************************************/
void SetUpScoringMatrices(CtrlType *ctrl)
{
  int i, j, k, size;
  float sum, localshift, i2jscore, j2iscore, isum, jsum;
  


  /* copy the frequency matrix of BLOSUM62 */
  size = PSSMWIDTH;
  ctrl->aafmat = iAllocMatrix(size, size, 0, "SetUpScoringMatrices: ctrl->aafmat");
  for (i=0; i<size; i++) 
      for (j=0; j<size; j++) 
	ctrl->aafmat[i][j] = FREQBLOSUM62[i*size+j];


  size = ctrl->size = strlen(AAORDER);
  ctrl->aaorder = strdup(AAORDER);

  ctrl->aasmat = fAllocMatrix(size, size, 0.0, "SetUpScoringMatrices: ctrl->aasmat");

  
  for (i=0; i<size; i++) 
      for (j=0; j<size; j++) 
	  ctrl->aasmat[i][j] = BLOSUM62[i*size+j];
    
  for (i=0; i<PSSMWIDTH; i++) {
      for (j=0; j<PSSMWIDTH; j++) { 
          i2jscore = j2iscore = isum = jsum = 0.0;
          for (k=0; k<PSSMWIDTH; k++) {
	      i2jscore += ctrl->aafmat[i][k]*BLOSUM62[j*size+k];
	      j2iscore += ctrl->aafmat[j][k]*BLOSUM62[i*size+k];
	      isum += ctrl->aafmat[i][k];
	      jsum += ctrl->aafmat[j][k];
          }
          ctrl->aasmat[i][j] = i2jscore/isum + j2iscore/jsum;
        }
  }
  
  




  /* Shift the scoring system */
  
  localshift = ctrl->shift;

  

  for (i=0; i<size; i++) {
    for (j=0; j<size; j++) {
      ctrl->aasmat[i][j] += localshift;
      
      /* Keep only three decimals of accuracy */
      ctrl->aasmat[i][j] = rint(1000*ctrl->aasmat[i][j])/1000.0;
    }
  }
  
  
}
