/*
 * struct.h
 *
 * This file holds the data for the sequences
 *
 *
 */


/*************************************************************************
* This data structure holds a sequence
**************************************************************************/
typedef struct {
  int len;
  int *sequence;
  int **pssm;
  int **psfm;
  char *name;
} SeqType;


/*************************************************************************
* This data structure holds a set of sequences
**************************************************************************/
typedef struct {
  int nseqs;
  SeqType *sequences;


  /* Family/class information for each sequence */
  char **family; 
  int nclasses, *class;
  char **classnames;


  /* AA sequence translation information */
  int nsymbols;
  int i2c[256];
  int c2i[256];
} SeqDBType;


/*************************************************************************
* This data structure stores the data set
**************************************************************************/
typedef struct {
  int nrows, ncols;
  int *rowptr, *rowind;
  float *rowval;

  int nclasses;
  int *class;
} MatType;




/*************************************************************************
* This data structure holds control information
**************************************************************************/
typedef struct {
  int PrgType;          /* The type of the program that I am running */

  int SimType;
    int SMType;  
    
  int kmerLen;
    float shift;          /* The value to be added to each p2pscore */
  float GOCost;         /* Gap opening cost */
  float GECost;         /* Gap extension cost */
    

  int seed;


  char *infile;
  char *infile1;
  char *infile2;

  /* Scoring matrices & related items */
  int size; 		/* The dimension of the AA scoring matrices */
  char *aaorder;	/* The order of the amino acids in the scoring matrices */
  float **aasmat;       /* The scoring matrix itself */

  int **aafmat;         /* The scaled raw frequency matrix to be used for filling empty PSSM rows */
} CtrlType;

